@@ -9,13 +9,13 @@ to disambiguate your commands so you can use Python reserved words and base
 types as your command names.  Without it, you can do a list_command or a
 for_command.
 
-You command then specifies its keyword arguments to indicate what has
+Your command then specifies its keyword arguments to indicate what has
 reasonable defaults and what is required.  Give a value to the option
 to indicate its default, and give a None setting to indicate it is required.
 A good way to read this is it is your commands "default settings" and None
 says "this option has no default setting".
 
-Here's an example from modargs tests:
+Here's an example from modargs tests::
 
    def test_command(port=8825, host='127.0.0.1', debug=1, sender=None, to=None,
                  subject=None, body=None, file=False):
@@ -23,11 +23,16 @@ Here's an example from modargs tests:
 You can see this has subject, body, sender, and to as required options (they 
 are None), and the rest have some default value.
 
-With this the argument parser will parse the users given arguments, and then
+With this the argument parser will parse the user's given arguments, and then
 call your command function with those as keyword arguments, but after it has
 fixed them up with the defaults you gave.  In the event that a user does
 not give a required option, modargs.args will abort with an error telling them.
 
+Options can also be specified multiple times to have their values collected as
+a list. For example, in the above method, we could invoke it with
+--to me@example.com --to you@example.com. When the test_command is invoked, to
+will have the value ['me@example.com', 'you@example.com'].
+
 Modargs's argument parser also accurately detects and parses integers, boolean
 values, strings, emails, single word values, and can handle trailing arguments
 after a -- argument.  This means you don't have to do conversion, it should be
@@ -55,12 +60,12 @@ S_WORD = lambda x, token:  ['word', token]
 S_EMAIL_ADDR = lambda x, token:  ['email', token]
 S_OPTION = lambda x, token:  ['option', token.split("-")[-1]]
 S_INT = lambda x, token:  ['int', int(token) ]
-S_BOOL = lambda x, token:  ['bool', bool(token) ]
+S_BOOL = lambda x, token:  ['bool', token in ['True', 'true', 'yes']]
 S_EMPTY = lambda x, token:  ['empty', '']
 S_STRING = lambda x, token:  ['string', token]
 S_TRAILING = lambda x, token:  ['trailing', None]
 
-class ArgumentError(Exception): 
+class ArgumentError(Exception):
     """Thrown when modargs.args encounters a command line format error."""
     pass
 
@@ -69,12 +74,12 @@ SCANNER = re.Scanner([
     (r"[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}", S_EMAIL_ADDR),
     (r"[0-9]+\.[0-9]+\.[0-9]+\.[0-9]", S_IP_ADDRESS),
     (r"-+[a-zA-Z0-9]+", S_OPTION),
-    (r"True", S_BOOL),
+    (r"True|False|true|false|yes|no", S_BOOL), # matching single letters causes problems
     (r"[0-9]+", S_INT),
     (r"--", S_TRAILING),
     (r"[a-z\-]+", S_WORD),
     (r"\s", S_EMPTY),
-    (r".+", S_STRING),
+    (r".+", S_STRING)
 ])
 
 
@@ -128,7 +133,16 @@ def option_production(data, tokens):
             data[opt] = True
         else:
             # this option is set to something else, so we'll grab that
-            data[opt] = match(tokens)
+            value = match(tokens)
+            # if we've encountered this opt before,
+            # construct a list or add it to an existing one
+            if opt in data:
+                try:
+                    data[opt].append(value)
+                except AttributeError:
+                    data[opt] = [data[opt], value]
+            else:
+                data[opt] = value
 
 
 def options_production(tokens):
@@ -164,11 +178,11 @@ def parse(argv):
     Tokenizes and then parses the command line as wither a command style or
     plain options style argument list.  It determines this by simply if the
     first argument is a 'word' then it's a command.  If not then it still
-    returns the first element of the tuple as None.  This means you can do:
+    returns the first element of the tuple as None.  This means you can do::
 
         command, options = args.parse(sys.argv[1:])
 
-    and if command==None then it was an option style, if not then it's a command 
+    and if `command==None` then it was an option style, if not then it's a command 
     to deal with.
     """
     tokens = tokenize(argv)
@@ -181,22 +195,63 @@ def parse(argv):
         # options only style
         return None, options_production(tokens)
 
+def function_for(mod, command, ending="_command"):
+    """
+    Return a reference to the function for a given command.
+    """
+    return mod.__dict__[command+ending]
 
 def determine_kwargs(function):
     """
     Uses the inspect module to figure out what the keyword arguments
     are and what they're defaults should be, then creates a dict with
-    that setup.  The results of determine_kwargs() is typically handed
-    to ensure_defaults().
+    that setup.  The results of `determine_kwargs()` is typically handed
+    to `ensure_defaults()`.
     """
     spec = inspect.getargspec(function)
     keys = spec[0]
     values = spec[-1]
     result = {}
-    for i in range(0, len(keys)):
-        result[keys[i]] = values[i]
+
+    if not values:
+        values = [] # otherwise len(None) causes exception
+
+    # If there are normal args, i.e. not keyword args, then the indexes
+    # have to be adjusted to match up kw args with their values.
+    #
+    # The n defaults in 'values' correspond to the last n arguments. If there
+    # are 5 keys and 3 values, then the 1st value corresponds to the 3rd key,
+    # or key[2] = value[0]
+    #
+    # Alternative to this is to raise an error if non-kw args are passed.
+    for i in range(0, len(values)):
+        keyindex = len(keys) - len(values) + i
+        result[keys[keyindex]] = values[i]
+
     return result
 
+def determine_kwdocs(function):
+    """
+    Retrieves docstrings written as comments in function definition.
+    """
+    keys = determine_kwargs(function).keys()
+    kwdocs = dict([(k, None) for k in keys])
+    source = inspect.getsourcelines(function)[0]
+    # TODO deal with multiple kwargs on the same line... should any comment apply to all of these? or none?
+    for l in source:
+        m = re.match("\s*([a-z]+)\s*=\s*(.+),?\s*#(.+)\s*", l)
+        if m:
+            arg, value, doc = m.groups()
+            if arg in keys:
+                kwdocs[arg] = doc.strip()
+        else:
+            # see if we can match kwargs
+            kwm = re.match("\s*\*\*([a-z]+)\s*#(.+)\s*", l)
+            if kwm:
+                kwarg, doc = kwm.groups()
+                kwdocs['kwargs'] = doc.strip()
+    return kwdocs
+
 def ensure_defaults(options, reqs):
     """
     Goes through the given options and the required ones and does the
@@ -218,7 +273,7 @@ def ensure_defaults(options, reqs):
 
 def command_module(mod, command, options, ending="_command"):
     """Takes a module, uses the command to run that function."""
-    function = mod.__dict__[command+ending]
+    function = function_for(mod, command, ending)
     kwargs = determine_kwargs(function)
     ensure_defaults(options, kwargs)
     try:
@@ -234,7 +289,9 @@ def available_help(mod, ending="_command"):
     for key in mod.__dict__:
         if key.endswith(ending):
             name = key.split(ending)[0]
-            help_text.append(name + ":\n" + mod.__dict__[key].__doc__)
+            docstring = mod.__dict__[key].__doc__
+            if docstring:
+                help_text.append(name + ":\n" + docstring)
 
     return help_text
 
@@ -268,7 +325,7 @@ def invalid_command_message(mod, exit_on_error):
     print "You must specify a valid command.  Try these: "
     print ", ".join(available_commands(mod))
 
-    if exit_on_error: 
+    if exit_on_error:
         sys.exit(1)
     else:
         return False
@@ -282,8 +339,8 @@ def parse_and_run_command(argv, mod, default_command=None, exit_on_error=True,
     give one then it runs that command.  If you don't specify a command,
     and they fail to give one then it prints an error.
 
-    On this error (failure to give a command) it will call sys.exit(1).
-    Set exit_on_error=False if you don't want this behavior, like if
+    On this error (failure to give a command) it will call `sys.exit(1)`.
+    Set `exit_on_error=False` if you don't want this behavior, like if
     you're doing a unit test.
     """
     try:
@@ -322,3 +379,69 @@ def load_module(name):
 
     return mod
 
+def help_command(prog, mod, default_command=None, on=False):
+    """Prints the help text generated by help_text."""
+    print help_text(prog, mod, default_command, on),
+
+def help_text(prog, mod, default_command=None, on=False):
+    """Generates help text based on command docstrings and named arguments."""
+    text = []
+    leading_spaces = "   "
+    if not on:
+        text.append("Available commands for %s are:" % prog)
+        for cmd in available_commands(mod):
+            text.append("%s%s" % (leading_spaces, cmd))
+        text.append("\nFor help on a particular command type, e.g., '%s help -on %s'" % (prog, cmd))
+
+    else:
+        command_help_text = help_for_command(mod, on)
+        if on == default_command:
+            prog_on = prog
+        else:
+            prog_on = "%s %s" % (prog, on)
+
+        dec = "=" * (11 + len(prog_on))
+        text.append(dec)
+        text.append("Help for '%s'" % prog_on)
+        text.append(dec)
+        if command_help_text:
+            for line in command_help_text.split("\n"):
+                text.append(leading_spaces + line.strip())
+
+        fn = function_for(mod, on)
+        kwargs = determine_kwargs(fn)
+        kwdocs = determine_kwdocs(fn)
+        if len(kwargs) > 0:
+            text.append(leading_spaces + "Arguments:")
+        for k in sorted(kwargs.keys()):
+            v = kwargs[k]
+            doc = kwdocs[k]
+            if doc:
+                # TODO wordwrap comments that are too long?
+                docs = "- %s" % doc
+            else:
+                docs = ""
+            if v is None:
+                arg_help_text = "%s%s %s,\n" % (leading_spaces*2, k, docs)
+                arg_help_text += "%s[required] e.g. '%s --%s <value>'\n" % (leading_spaces*4, prog_on, k)
+            else:
+                arg_help_text = "%s%s %s\n" % (leading_spaces*2, k, docs)
+                arg_help_text += "%s[optional, defaults to '%s']" % (leading_spaces*4, v)
+
+                if len(str(v)) > 8:
+                    # put example on its own line
+                    arg_help_text += "\n" + (leading_spaces * 4)
+
+                if ' ' in str(v):
+                    v_str = "\"%s\"" % v
+                else:
+                    v_str = v
+                arg_help_text += " e.g. '%s --%s %s'\n" % (prog_on, k, v_str)
+
+            text.append(arg_help_text + "\n")
+
+        if kwdocs.has_key('kwargs'):
+            text.append("%sKeyword Arguments:\n" % (leading_spaces))
+            text.append("%s%s\n\n" % (leading_spaces*2, kwdocs['kwargs']))
+
+    return "\n".join(text)
